//============================================================================
// 635 WinTest Code.
// Copyright 2001, Crystalfontz America, Inc. Written by Brent A. Crosby
// www.crystalfontz.com, brent@crystalfontz.com
//============================================================================
#include        <stdlib.h>
#include        <termios.h>
#include        <unistd.h>
#include        <fcntl.h>
#include        <errno.h>
#include        <ctype.h>
#include	<stdio.h>
#include	"typedefs.h"
#include	"serial.h"

#define FALSE   0
#define TRUE    1

// serial device name
const char *serial_port_names[]=
	{"ttyS0", "ttyS1", "ttyS2", "ttyS3", "ttyS4",
	"ttyS5", "ttyS6", "ttyS7", "ttyS8", "ttyS9"};

// com port handle
int handle;

// data buffering
#define RECEIVEBUFFERSIZE 4096
ubyte SerialReceiveBuffer[RECEIVEBUFFERSIZE];
dword ReceiveBufferHead;
dword ReceiveBufferTail;
dword ReceiveBufferTailPeek;

//------------------------------------------------------------------------------
int Serial_Init(int port, int baud_rate)
{
	char devname[512];
	int brate;
        struct termios term; 

	//make device name string
	sprintf(devname, "/dev/%s", serial_port_names[port]);
	
	//open device
	handle = open(devname, 2);
		printf("%d=open (%s, %d)\n",handle, devname, 2 );
	if (handle <= 0)
	{
		//error opening port
		printf("unable to open device\n");
		return(1);
	}

	//get baud rate
	switch (baud_rate)
	{
		case 1200:
			brate=B1200;
			break;
		case 1800:
			brate=B1800;
			break;
		case 2400:
			brate=B2400;
			break;
		case 4800:
			brate=B4800;
			break;
		case 9600:
			brate=B9600;
			break;
		case 19200:
			brate=B19200;
			break;
		case 38400:
			brate=B38400;
			break;
		default:
			printf("invalid baud rate: %d\n", baud_rate);
			return(2);
	}

	//get device struct
        if (tcgetattr(handle, &term) != 0)
        {
                printf("tcgetattr failed\n");
                return(3);
        }
	printf("tcgetattr(%d, %d )\n", handle, &term  );
        //input modes 
        term.c_iflag &= ~(IGNBRK|BRKINT|PARMRK|INPCK|ISTRIP|INLCR|IGNCR|ICRNL
                        |IUCLC|IXON|IXANY|IXOFF|IMAXBEL);
        term.c_iflag |= IGNPAR;

        //output modes 
        term.c_oflag &= ~(OPOST|OLCUC|ONLCR|OCRNL|ONOCR|ONLRET|OFILL
                        |OFDEL|NLDLY|CRDLY|TABDLY|BSDLY|VTDLY|FFDLY);
        term.c_oflag |= NL0|CR0|TAB0|BS0|VT0|FF0;

        //control modes
        term.c_cflag &= ~(CSIZE|PARENB|CRTSCTS|PARODD|HUPCL);
        term.c_cflag |= CREAD|CS8|CSTOPB|CLOCAL;

        //local modes 
        term.c_lflag &= ~(ISIG|ICANON|IEXTEN|ECHO|FLUSHO|PENDIN);
        term.c_lflag |= NOFLSH;

        //set baud rate
        cfsetospeed(&term, brate);
        cfsetispeed(&term, brate);

	//set new device settings
        if (tcsetattr(handle, TCSANOW, &term)  != 0)
        {
		printf("tcsetattr failed\n");
		return(4);
        }
}
//------------------------------------------------------------------------------
void Uninit_Serial()
{
	close(handle);
	handle=(int)NULL;
}
//------------------------------------------------------------------------------
void SendByte(unsigned char datum)
{
	dword bytes_written;
	bytes_written=0;

	if(handle)
		if(write(handle, &datum, 1) != 1)
			printf("SendByte() failed\n");
}
//------------------------------------------------------------------------------
//Gets incoming data and puts it into SerialReceiveBuffer[].
void Sync_Read_Buffer(ubyte expected_bytes)
{
	ubyte Incoming[4096];
	dword BytesRead;
//	COMSTAT status;
	dword errors;
	dword i;

	if(!handle)
		return;

	//get data
	BytesRead = read(handle, &Incoming, 4096);

	//Read the incoming ubyte, store it.
	for(i=0; i < BytesRead; i++)
	{
		SerialReceiveBuffer[ReceiveBufferHead] = Incoming[i];

		//Increment the pointer and wrap if needed.
		ReceiveBufferHead++;
		if (RECEIVEBUFFERSIZE <= ReceiveBufferHead)
			ReceiveBufferHead=0;
	}
 }
/*---------------------------------------------------------------------------*/
dword BytesAvail(void)
{
	//LocalReceiveBufferHead and return_value are a signed variables.
	int LocalReceiveBufferHead;
	int return_value;
	
	//Get a register copy of the head pointer, since an interrupt that
	//modifies the head pointer could screw up the value. Convert it to
	//our signed local variable.
	LocalReceiveBufferHead = ReceiveBufferHead;
	if ((return_value = (LocalReceiveBufferHead - (int)ReceiveBufferTail)) < 0)
		return_value+=RECEIVEBUFFERSIZE;

	return(return_value);
}
/*---------------------------------------------------------------------------*/
ubyte GetByte(void)
{
	dword LocalReceiveBufferTail;
	dword LocalReceiveBufferHead;
	ubyte return_byte;

	//Get a register copy of the tail pointer for speed and size.
	LocalReceiveBufferTail=ReceiveBufferTail;
	
	//Get a register copy of the head pointer, since an interrupt that
	//modifies the head pointer could screw up the value.
	LocalReceiveBufferHead=ReceiveBufferHead;

	
	//See if there are any more bytes available.
	if(LocalReceiveBufferTail!=LocalReceiveBufferHead)
	{
		//There is at least one more ubyte.
		return_byte=SerialReceiveBuffer[LocalReceiveBufferTail];

		//Increment the pointer and wrap if needed.
		LocalReceiveBufferTail++;
		if(RECEIVEBUFFERSIZE<=LocalReceiveBufferTail)
			LocalReceiveBufferTail=0;

		//Update the real ReceiveBufferHead with our register copy. Make sure
		//the ISR does not get serviced during the transfer.
		ReceiveBufferTail=LocalReceiveBufferTail;
	}
	
	return(return_byte);
}
/*---------------------------------------------------------------------------*/
dword PeekBytesAvail(void)
{
	//LocalReceiveBufferHead and return_value are a signed variables.
	int LocalReceiveBufferHead;
	int return_value;
	
	//Get a register copy of the head pointer, since an interrupt that
	//modifies the head pointer could screw up the value. Convert it to
	//our signed local variable.
	LocalReceiveBufferHead=ReceiveBufferHead;
	if((return_value=(LocalReceiveBufferHead-(int)ReceiveBufferTailPeek))<0)
		return_value+=RECEIVEBUFFERSIZE;
	return(return_value);
}
/*---------------------------------------------------------------------------*/
void Sync_Peek_Pointer(void)
{
	ReceiveBufferTailPeek=ReceiveBufferTail;
}
/*---------------------------------------------------------------------------*/
void AcceptPeekedData(void)
{
	ReceiveBufferTail=ReceiveBufferTailPeek;
}
/*---------------------------------------------------------------------------*/
ubyte PeekByte(void)
{
	int LocalReceiveBufferTailPeek;
	int LocalReceiveBufferHead;
	ubyte return_byte;

	//Get a register copy of the tail pointer for speed and size.
	LocalReceiveBufferTailPeek=ReceiveBufferTailPeek;
	
	//Get a register copy of the head pointer, since an interrupt that
	//modifies the head pointer could screw up the value.
	LocalReceiveBufferHead=ReceiveBufferHead;
	
	//See if there are any more bytes available.
	if(LocalReceiveBufferTailPeek!=LocalReceiveBufferHead)
	{
		//There is at least one more ubyte.
		return_byte=SerialReceiveBuffer[LocalReceiveBufferTailPeek];

		//Increment the pointer and wrap if needed.
		LocalReceiveBufferTailPeek++;
		if(RECEIVEBUFFERSIZE<=LocalReceiveBufferTailPeek)
			LocalReceiveBufferTailPeek=0;

		//Update the real ReceiveBufferHead with our register copy. Make sure
		//the ISR does not get serviced during the transfer.
		ReceiveBufferTailPeek=LocalReceiveBufferTailPeek;
	}
	return(return_byte);
}
/*---------------------------------------------------------------------------*/
